
#ifndef BL_TASK_H_
#define BL_TASK_H_

#include "timeSliceOffset.h"

// extern STR_TimeSliceOffset g_SOE_flag;
// extern void Task_SOE_flag(void);


void task_Init(void);



void beginStartControlTask(void);
void beginSoftStartTask(void);
// void beginHYconfigMode(void);
// void uartTaskInit(void);

void startShortCircuitProtection(void);
void stopShortCircuitProtection(void);
void startExcessiveLoadProtection(void);
void startSoftShortCircuitProtection(void);
void startEXCHGSCProtection(void);

void chargRunLed(uint8_t mode);


#endif
